@@ -1,5 +1,5 @@
 /*
-//@line 42 "/export/home/mozilla/uild/firefox-1.5.0.12/src/mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in"
+//@line 42 "/export/home/mozilla/uild/firefox-2.0.0.3/src/mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in"
 */
 
 /* This file implements the nsIHelperAppLauncherDialog interface.
@@ -99,9 +99,9 @@ nsUnknownContentTypeDialog.prototype = {
           if (aFolderType != "Downloads")
             throw "ASSERTION FAILED: folder type should be 'Desktop' or 'Downloads'";
         
-//@line 147 "/export/home/mozilla/uild/firefox-1.5.0.12/src/mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in"
+//@line 147 "/export/home/mozilla/uild/firefox-2.0.0.3/src/mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in"
           return "Home";
-//@line 150 "/export/home/mozilla/uild/firefox-1.5.0.12/src/mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in"
+//@line 150 "/export/home/mozilla/uild/firefox-2.0.0.3/src/mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in"
         }
         
         function getDownloadsFolder(aFolder)
@@ -191,22 +191,43 @@ nsUnknownContentTypeDialog.prototype = {
         result = picker.file;
 
         if (result) {
+          try {
+            // Remove the file so that it's not there when we ensure non-existence later;
+            // this is safe because for the file to exist, the user would have had to
+            // confirm that he wanted the file overwritten.
+            if (result.exists())
+              result.remove(false);
+          }
+          catch (e) { }
           var newDir = result.parent;
           prefs.setComplexValue("browser.download.dir", Components.interfaces.nsILocalFile, newDir);
+          result = this.validateLeafName(newDir, result.leafName, null);
         }
       }
       return result;
     },
-    
+
+    /**
+     * Ensures that a local folder/file combination does not already exist in
+     * the file system (or finds such a combination with a reasonably similar
+     * leaf name), creates the corresponding file, and returns it.
+     *
+     * @param   aLocalFile
+     *          the folder where the file resides
+     * @param   aLeafName
+     *          the string name of the file (may be empty if no name is known,
+     *          in which case a name will be chosen)
+     * @param   aFileExt
+     *          the extension of the file, if one is known; this will be ignored
+     *          if aLeafName is non-empty
+     * @returns nsILocalFile
+     *          the created file
+     */
     validateLeafName: function (aLocalFile, aLeafName, aFileExt)
     {
       if (!aLocalFile || !aLocalFile.exists())
         return null;
 
-      // Remove any leading periods, since we don't want to save hidden files
-      // automatically.
-      aLeafName = aLeafName.replace(/^\.+/, "");
-
       if (aLeafName == "")
         aLeafName = "unnamed" + (aFileExt ? "." + aFileExt : "");
       aLocalFile.append(aLeafName);
@@ -223,12 +244,14 @@ nsUnknownContentTypeDialog.prototype = {
       return aLocalFile;
     },
 
+    /**
+     * Generates and returns a uniquely-named file from aLocalFile.  If
+     * aLocalFile does not exist, it will be the file returned; otherwise, a
+     * file whose name is similar to that of aLocalFile will be returned.
+     */
     makeFileUnique: function (aLocalFile)
     {
       try {
-        // Since we're automatically downloading, we don't get the file picker's 
-        // logic to check for existing files, so we need to do that here.
-        //
         // Note - this code is identical to that in 
         //   toolkit/content/contentAreaUtils.js.
         // If you are updating this code, update that code too! We can't share code
@@ -352,32 +375,50 @@ nsUnknownContentTypeDialog.prototype = {
       var iconString = "moz-icon://" + fname + "?size=16&contentType=" + this.mLauncher.MIMEInfo.MIMEType;
       this.dialogElement("contentTypeImage").setAttribute("src", iconString);
 
-      this.initAppAndSaveToDiskValues();
-
-      // Initialize "always ask me" box. This should always be disabled
-      // and set to true for the ambiguous type application/octet-stream.
-      // We don't also check for application/x-msdownload here since we
-      // want users to be able to autodownload .exe files. 
-      var rememberChoice = this.dialogElement("rememberChoice");
-
-//@line 428 "/export/home/mozilla/uild/firefox-1.5.0.12/src/mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in"
+      // if always-save and is-executable and no-handler
+      // then set up simple ui
       var mimeType = this.mLauncher.MIMEInfo.MIMEType;
-      if (mimeType == "application/octet-stream" || 
-          mimeType == "application/x-msdownload" ||
-          this.mLauncher.targetFile.isExecutable()) {
-        rememberChoice.checked = false;
-        rememberChoice.disabled = true;
+      var shouldntRememberChoice = (mimeType == "application/octet-stream" || 
+                                    mimeType == "application/x-msdownload" ||
+                                    this.mLauncher.targetFile.isExecutable());
+      if (shouldntRememberChoice && !this.openWithDefaultOK()) {
+        // hide featured choice 
+        this.mDialog.document.getElementById("normalBox").collapsed = true;
+        // show basic choice 
+        this.mDialog.document.getElementById("basicBox").collapsed = false;
+        // change button labels
+        this.mDialog.document.documentElement.getButton("accept").label = this.dialogElement("strings").getString("unknownAccept.label");
+        this.mDialog.document.documentElement.getButton("cancel").label = this.dialogElement("strings").getString("unknownCancel.label");
+        // hide other handler
+        this.mDialog.document.getElementById("openHandler").collapsed = true;
+        // set save as the selected option
+        this.dialogElement("mode").selectedItem = this.dialogElement("save");
       }
       else {
-        rememberChoice.checked = !this.mLauncher.MIMEInfo.alwaysAskBeforeHandling;
-      }
-      this.toggleRememberChoice(rememberChoice);
+        this.initAppAndSaveToDiskValues();
 
-      // XXXben - menulist won't init properly, hack. 
-      var openHandler = this.dialogElement("openHandler");
-      openHandler.parentNode.removeChild(openHandler);
-      var openHandlerBox = this.dialogElement("openHandlerBox");
-      openHandlerBox.appendChild(openHandler);
+        // Initialize "always ask me" box. This should always be disabled
+        // and set to true for the ambiguous type application/octet-stream.
+        // We don't also check for application/x-msdownload here since we
+        // want users to be able to autodownload .exe files. 
+        var rememberChoice = this.dialogElement("rememberChoice");
+
+//@line 471 "/export/home/mozilla/uild/firefox-2.0.0.3/src/mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in"
+        if (shouldntRememberChoice) {
+          rememberChoice.checked = false;
+          rememberChoice.disabled = true;
+        }
+        else {
+          rememberChoice.checked = !this.mLauncher.MIMEInfo.alwaysAskBeforeHandling;
+        }
+        this.toggleRememberChoice(rememberChoice);
+
+        // XXXben - menulist won't init properly, hack. 
+        var openHandler = this.dialogElement("openHandler");
+        openHandler.parentNode.removeChild(openHandler);
+        var openHandlerBox = this.dialogElement("openHandlerBox");
+        openHandlerBox.appendChild(openHandler);
+      }
 
       this.mDialog.setTimeout("dialog.postShowCallback()", 0);
       
@@ -495,12 +536,12 @@ nsUnknownContentTypeDialog.prototype = {
         var result;
 
         // The checking is different on Windows...
-//@line 576 "/export/home/mozilla/uild/firefox-1.5.0.12/src/mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in"
+//@line 617 "/export/home/mozilla/uild/firefox-2.0.0.3/src/mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in"
             // On other platforms, default is Ok if there is a default app.
             // Note that nsIMIMEInfo providers need to ensure that this holds true
             // on each platform.
         return this.mLauncher.MIMEInfo.hasDefaultHandler;
-//@line 581 "/export/home/mozilla/uild/firefox-1.5.0.12/src/mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in"
+//@line 622 "/export/home/mozilla/uild/firefox-2.0.0.3/src/mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in"
     },
     
     // Set "default" application description field.
@@ -521,9 +562,9 @@ nsUnknownContentTypeDialog.prototype = {
 
     // getPath:
     getPath: function (aFile) {
-//@line 604 "/export/home/mozilla/uild/firefox-1.5.0.12/src/mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in"
+//@line 645 "/export/home/mozilla/uild/firefox-2.0.0.3/src/mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in"
       return aFile.path;
-//@line 606 "/export/home/mozilla/uild/firefox-1.5.0.12/src/mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in"
+//@line 647 "/export/home/mozilla/uild/firefox-2.0.0.3/src/mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in"
     },
 
     // initAppAndSaveToDiskValues:
@@ -591,9 +632,9 @@ nsUnknownContentTypeDialog.prototype = {
       }
       
       // otherHandler is always disabled on Mac
-//@line 677 "/export/home/mozilla/uild/firefox-1.5.0.12/src/mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in"
+//@line 718 "/export/home/mozilla/uild/firefox-2.0.0.3/src/mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in"
       otherHandler.nextSibling.hidden = otherHandler.nextSibling.nextSibling.hidden = false;
-//@line 679 "/export/home/mozilla/uild/firefox-1.5.0.12/src/mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in"
+//@line 720 "/export/home/mozilla/uild/firefox-2.0.0.3/src/mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in"
       this.updateOKButton();
     },
 
@@ -704,6 +745,7 @@ nsUnknownContentTypeDialog.prototype = {
     updateHelperAppPref: function() {
       var ha = new this.mDialog.HelperApps();
       ha.updateTypeInfo(this.mLauncher.MIMEInfo);
+      ha.destroy();
     },
     
     // onOK:
@@ -746,7 +788,7 @@ nsUnknownContentTypeDialog.prototype = {
           // for the file to be saved to to pass to |saveToDisk| - otherwise
           // we must ask the user to pick a save name.
 
-//@line 845 "/export/home/mozilla/uild/firefox-1.5.0.12/src/mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in"
+//@line 887 "/export/home/mozilla/uild/firefox-2.0.0.3/src/mozilla/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in"
           this.mLauncher.saveToDisk(null, false);
         }
         else
